function Build_SR_FBA_Problem()

% ***************************************** User Area Starts here ********************************************

% Load the required E. Coli Regulatory-Metabolic model according to your choice
% If you want to use model version 2, mark the next line and unmark the following 
load('SRFBA_model_V1.mat');
%load('SRFBA_model_V2.mat');

% In the following function you should set the environment as you wish
setYourEnv;

% Set the following 4 parameters according to your needs
outputFileName = 'cplex_input.txt';

% Set the network element type you would like to maximize/minimize
% One of: 
% 'gene' => maximize or minimize the gene's boolean.
% 'protein' => maximize or minimize the protein's boolean.
% 'reaction' => maximize or minimize the reaction's flux value
objective_element = 'reaction'; 

% The following index is the index of the network element you would like to maximize/minimize.
% The index meaning should be taken from:
% Genes - 'iMC1010v1.xls' or 'iMC1010v2.xls' (http://gcrg.ucsd.edu/organisms/ecoli/regulation/iMC1010v1.xls)
% Proteins - 'iJR904(7-28-04).xls' (http://gcrg.ucsd.edu/organisms/ecoli/metabrxns/iJR904(7-28-04).xls)
% Reactions - 'iJR904(7-28-04).xls' (http://gcrg.ucsd.edu/organisms/ecoli/metabrxns/iJR904(7-28-04).xls)
objective_index = 256; % 256 stands for the biomass reaction
maximize_or_minimize = 'maximize'; % One of: 'maximize' or 'minimize' 

% ***************************************** User Area Ends here ********************************************

BuildMIPProblem;        
CreateCPlexInputFile(outputFileName, objective_element, objective_index, maximize_or_minimize);

PrintProblemStats;

% All names in this function are taken from the file 'iJR904(7-28-04).xls' published in this link:
% http://gcrg.ucsd.edu/organisms/ecoli/metabrxns/iJR904(7-28-04).xls
function setYourEnv()

global reactions;

% Default metabolites are set here
reactions.reactionConstrainsLow(959) = -1000; % co2(e)
reactions.reactionConstrainsLow(974) = -1000; % fe2(e)
reactions.reactionConstrainsLow(1002) = -1000; % h(e)
reactions.reactionConstrainsLow(1003) = -1000; % h2o(e)
reactions.reactionConstrainsLow(1011) = -1000; % k(e)
reactions.reactionConstrainsLow(1029) = -1000; % na1(e)
reactions.reactionConstrainsLow(1032) = -1000; % nh4(e)
reactions.reactionConstrainsLow(1040) = -1000; % pi(e)
reactions.reactionConstrainsLow(1051) = -1000; % so4(e)

% Oxygen concentration is set here
reactions.reactionConstrainsLow(1036) = -1000; % o2_ext

% All other metabolites consentration are set here
% A negative value represents an external flux that can ENTER the model
reactions.reactionConstrainsLow(931) = 0; %   '12ppd-s_ext'
reactions.reactionConstrainsLow(932) = 0; %   '15dap_ext'		
reactions.reactionConstrainsLow(933) = 0; %   '26dap-m_ext'
reactions.reactionConstrainsLow(934) = 0; %   '2ddglcn_ext'
reactions.reactionConstrainsLow(935) = 0; %   '3hcinnm_ext'
reactions.reactionConstrainsLow(936) = 0; %   '3hpppn_ext'
reactions.reactionConstrainsLow(937) = 0; %   '4abut_ext'
reactions.reactionConstrainsLow(938) = 0; %   'ac_ext'
reactions.reactionConstrainsLow(939) = 0; %   'acac_ext'
reactions.reactionConstrainsLow(940) = 0; %   'acald_ext'
reactions.reactionConstrainsLow(941) = 0; %   'acgam_ext'
reactions.reactionConstrainsLow(942) = 0; %   'acmana_ext'
reactions.reactionConstrainsLow(943) = 0; %   'acnam_ext'
reactions.reactionConstrainsLow(944) = 0; %   'ade_ext'
reactions.reactionConstrainsLow(945) = 0; %   'adn_ext'
reactions.reactionConstrainsLow(946) = 0; %   'akg_ext'
reactions.reactionConstrainsLow(947) = 0; %   'ala-d_ext'
reactions.reactionConstrainsLow(948) = 0; %   'ala-l_ext'
reactions.reactionConstrainsLow(949) = 0; %   'alltn_ext'
reactions.reactionConstrainsLow(950) = 0; %   'amp_ext'
reactions.reactionConstrainsLow(951) = 0; %   'arab-l_ext'
reactions.reactionConstrainsLow(952) = 0; %   'arg-l_ext'
reactions.reactionConstrainsLow(953) = 0; %   'asn-l_ext'
reactions.reactionConstrainsLow(954) = 0; %   'asp-l_ext'
reactions.reactionConstrainsLow(955) = 0; %   'but_ext'
reactions.reactionConstrainsLow(956) = 0; %   'cbl1_ext'
reactions.reactionConstrainsLow(957) = 0; %   'chol_ext'
reactions.reactionConstrainsLow(958) = 0; %   'cit_ext'
reactions.reactionConstrainsLow(960) = 0; %   'crn_ext'
reactions.reactionConstrainsLow(961) = 0; %   'csn_ext'
reactions.reactionConstrainsLow(962) = 0; %   'cynt_ext'
reactions.reactionConstrainsLow(963) = 0; %   'cys-l_ext'
reactions.reactionConstrainsLow(964) = 0; %   'cytd_ext'
reactions.reactionConstrainsLow(965) = 0; %   'dad-2_ext'
reactions.reactionConstrainsLow(966) = 0; %   'dcyt_ext'
reactions.reactionConstrainsLow(967) = 0; %   'dgsn_ext'
reactions.reactionConstrainsLow(968) = 0; %   'dha_ext'
reactions.reactionConstrainsLow(969) = 0; %   'din_ext'
reactions.reactionConstrainsLow(970) = 0; %   'dms_ext'
reactions.reactionConstrainsLow(971) = 0; %   'dmso_ext'
reactions.reactionConstrainsLow(972) = 0; %   'duri_ext'
reactions.reactionConstrainsLow(973) = 0; %   'etoh_ext'
reactions.reactionConstrainsLow(975) = 0; %   'for_ext'
reactions.reactionConstrainsLow(976) = 0; %   'fru_ext'
reactions.reactionConstrainsLow(977) = 0; %   'fuc-l_ext'
reactions.reactionConstrainsLow(978) = 0; %   'fuc1p-l_ext'
reactions.reactionConstrainsLow(979) = 0; %   'fum_ext'
reactions.reactionConstrainsLow(980) = 0; %   'g6p_ext'
reactions.reactionConstrainsLow(981) = 0; %   'gal_ext'
reactions.reactionConstrainsLow(982) = 0; %   'galct-d_ext'
reactions.reactionConstrainsLow(983) = 0; %   'galctn-d_ext'
reactions.reactionConstrainsLow(984) = 0; %   'galt_ext'
reactions.reactionConstrainsLow(985) = 0; %   'galur_ext'
reactions.reactionConstrainsLow(986) = 0; %   'gam_ext'
reactions.reactionConstrainsLow(987) = 0; %   'gbbtn_ext'
reactions.reactionConstrainsLow(988) = -10; %   'glc-d_ext'
reactions.reactionConstrainsLow(989) = 0; %   'glcn_ext'
reactions.reactionConstrainsLow(990) = 0; %   'glcr_ext'
reactions.reactionConstrainsLow(991) = 0; %   'glcur_ext'
reactions.reactionConstrainsLow(992) = 0; %   'gln-l_ext'
reactions.reactionConstrainsLow(993) = 0; %   'glu-l_ext'
reactions.reactionConstrainsLow(994) = 0; %   'gly_ext'
reactions.reactionConstrainsLow(995) = 0; %   'glyald_ext'
reactions.reactionConstrainsLow(996) = 0; %   'glyb_ext'
reactions.reactionConstrainsLow(997) = 0; %   'glyc_ext'
reactions.reactionConstrainsLow(998) = 0; %   'glyc3p_ext'
reactions.reactionConstrainsLow(999) = 0; %   'glyclt_ext'
reactions.reactionConstrainsLow(1000) = 0; %  'gsn_ext'
reactions.reactionConstrainsLow(1001) = 0; %  'gua_ext'
reactions.reactionConstrainsLow(1004) = 0; %  'hdca_ext'
reactions.reactionConstrainsLow(1005) = 0; %  'his-l_ext'
reactions.reactionConstrainsLow(1006) = 0; %  'hxan_ext'
reactions.reactionConstrainsLow(1007) = 0; %  'idon-l_ext'
reactions.reactionConstrainsLow(1008) = 0; %  'ile-l_ext'
reactions.reactionConstrainsLow(1009) = 0; %  'indole_ext'
reactions.reactionConstrainsLow(1010) = 0; %  'ins_ext'
reactions.reactionConstrainsLow(1012) = 0; %  'lac-d_ext'
reactions.reactionConstrainsLow(1013) = 0; %  'lac-l_ext'
reactions.reactionConstrainsLow(1014) = 0; %  'lcts_ext'
reactions.reactionConstrainsLow(1015) = 0; %  'leu-l_ext'
reactions.reactionConstrainsLow(1016) = 0; %  'lys-l_ext'
reactions.reactionConstrainsLow(1017) = 0; %  'mal-l_ext'
reactions.reactionConstrainsLow(1018) = 0; %  'malt_ext'
reactions.reactionConstrainsLow(1019) = 0; %  'malthx_ext'
reactions.reactionConstrainsLow(1020) = 0; %  'maltpt_ext'
reactions.reactionConstrainsLow(1021) = 0; %  'malttr_ext'
reactions.reactionConstrainsLow(1022) = 0; %  'maltttr_ext'
reactions.reactionConstrainsLow(1023) = 0; %  'man_ext'
reactions.reactionConstrainsLow(1024) = 0; %  'man6p_ext'
reactions.reactionConstrainsLow(1025) = 0; %  'melib_ext'
reactions.reactionConstrainsLow(1026) = 0; %  'met-d_ext'
reactions.reactionConstrainsLow(1027) = 0; %  'met-l_ext'
reactions.reactionConstrainsLow(1028) = 0; %  'mnl_ext'
reactions.reactionConstrainsLow(1030) = 0; %  'nac_ext'
reactions.reactionConstrainsLow(1031) = 0; %  'nad_ext'
reactions.reactionConstrainsLow(1033) = 0; %  'nmn_ext'
reactions.reactionConstrainsLow(1034) = 0; %  'no2_ext'
reactions.reactionConstrainsLow(1035) = 0; %  'no3_ext'
reactions.reactionConstrainsLow(1037) = 0; %  'ocdca_ext'
reactions.reactionConstrainsLow(1038) = 0; %  'orn_ext'
reactions.reactionConstrainsLow(1039) = 0; %  'phe-l_ext'
reactions.reactionConstrainsLow(1041) = 0; %  'pnto-r_ext'
reactions.reactionConstrainsLow(1042) = 0; %  'pppn_ext'
reactions.reactionConstrainsLow(1043) = 0; %  'pro-l_ext'
reactions.reactionConstrainsLow(1044) = 0; %  'ptrc_ext'
reactions.reactionConstrainsLow(1045) = 0; %  'pyr_ext'
reactions.reactionConstrainsLow(1046) = 0; %  'rib-d_ext'
reactions.reactionConstrainsLow(1047) = 0; %  'rmn_ext'
reactions.reactionConstrainsLow(1048) = 0; %  'sbt-d_ext'
reactions.reactionConstrainsLow(1049) = 0; %  'ser-d_ext'
reactions.reactionConstrainsLow(1050) = 0; %  'ser-l_ext'
reactions.reactionConstrainsLow(1052) = 0; %  'spmd_ext'
reactions.reactionConstrainsLow(1053) = 0; %  'succ_ext'
reactions.reactionConstrainsLow(1054) = 0; %  'sucr_ext'
reactions.reactionConstrainsLow(1055) = 0; %  'tartr-l_ext'
reactions.reactionConstrainsLow(1056) = 0; %  'taur_ext'
reactions.reactionConstrainsLow(1057) = 0; %  'thm_ext'
reactions.reactionConstrainsLow(1058) = 0; %  'thr-l_ext'
reactions.reactionConstrainsLow(1059) = 0; %  'thymd_ext'
reactions.reactionConstrainsLow(1060) = 0; %  'tma_ext'
reactions.reactionConstrainsLow(1061) = 0; %  'tmao_ext'
reactions.reactionConstrainsLow(1062) = 0; %  'tre_ext'
reactions.reactionConstrainsLow(1063) = 0; %  'trp-l_ext'
reactions.reactionConstrainsLow(1064) = 0; %  'tsul_ext'
reactions.reactionConstrainsLow(1065) = 0; %  'ttdca_ext'
reactions.reactionConstrainsLow(1066) = 0; %  'tyr-l_ext'
reactions.reactionConstrainsLow(1067) = 0; %  'ura_ext'
reactions.reactionConstrainsLow(1068) = 0; %  'urea_ext'
reactions.reactionConstrainsLow(1069) = 0; %  'uri_ext'
reactions.reactionConstrainsLow(1070) = 0; %  'val-l_ext'
reactions.reactionConstrainsLow(1071) = 0; %  'xan_ext'
reactions.reactionConstrainsLow(1072) = 0; %  'xtsn_ext'
reactions.reactionConstrainsLow(1073) = 0; %  'xyl-d_ext'
reactions.reactionConstrainsLow(1078) = 0; %  'h2s_ext'
reactions.reactionConstrainsLow(1079) = 0; %  '5dglcn_ext'
reactions.reactionConstrainsLow(1080) = 0; %  'btn_ext'
reactions.reactionConstrainsLow(1081) = 0; %  'cbi_ext'
reactions.reactionConstrainsLow(1082) = 0; %  'h2o2_ext'
reactions.reactionConstrainsLow(1083) = 0; %  'ppa_ext'
reactions.reactionConstrainsLow(1084) = 0; %  'thym_ext'

function BuildMIPProblem()

global genes;
global proteins;
global reactions;
global mipMatrix;
global mipRowLB;
global mipRowUB;
global mipColumnLB;
global mipColumnUB;
global mipVariableType;
global lastVariableIndex;

format long;

matrixSize = size(reactions.stoichMatrix);
lastVariableIndex = matrixSize(2);
mipVariableType = [];

% ****************** Build metabolite constrains - copy the Stoichiometric matrix from the reactions DB ****************** 
mipMatrix = sparse(reactions.stoichMatrix); % Full do the oppiste operation - converts sparse matrix to full
mipColumnLB = [];
mipColumnUB = [];

for i=1:matrixSize(2)
    % Set all fluxes variables to Real variables
    % The reactions indices are as their index in the reactions global variable
    mipVariableType = [mipVariableType 'C'];
    mipColumnLB = [mipColumnLB ; reactions.reactionConstrainsLow(i)];
    mipColumnUB = [mipColumnUB ; reactions.reactionConstrainsHigh(i)];
end
mipRowLB = zeros(matrixSize(1),1);
mipRowUB = zeros(matrixSize(1),1);

gene_variables;
gene_equations; % reaction predicats are built here when necessary
protein_variables;
protein_equations;
reaction_variables;
reaction_equations;
reaction_bounds_equations;

function gene_variables()

global genes;
global lastVariableIndex;

% first build the boolean accosiated with each gene
genes.geneBoolVariableIndex = zeros(length(genes.geneName),1);

for i=1:length(genes.geneName)
    lastVariableIndex = lastVariableIndex + 1;
    addBooleanMIPVariable;
    genes.geneBoolVariableIndex(i) = lastVariableIndex;
end

function gene_equations() 

global genes;
global lastVariableIndex;

for i=1:length(genes.geneName)
    rule_data_struct = genes.regulationRuleDS{i};
    lastVariableIndex = ConvertRuleToEquations(rule_data_struct, 'gene_mode', i, lastVariableIndex);   
end

function protein_variables()

global proteins;
global lastVariableIndex;

% ****************** build proteins variables and equations ******************
% first build the boolean accosiated with each protein
proteins.proteinBoolVariableIndex = zeros(length(proteins.proteinName),1);

for i=1:length(proteins.proteinName)
    lastVariableIndex = lastVariableIndex + 1;
    addBooleanMIPVariable;
    proteins.proteinBoolVariableIndex(i) = lastVariableIndex;
end

function protein_equations()

global proteins;
global lastVariableIndex;

for i=1:length(proteins.proteinName)
    rule_data_struct = proteins.proteinAssociationWithGenesDS{i};
    lastVariableIndex = ConvertRuleToEquations(rule_data_struct, 'protein_mode', i, lastVariableIndex);   
end

function reaction_variables()

global reactions;
global lastVariableIndex;

% first build the boolean accosiated with each reaction
reactions.reactionBoolVariableIndex = zeros(length(reactions.reactionName),1);

for i=1:length(reactions.reactionName)
    lastVariableIndex = lastVariableIndex + 1;
    addBooleanMIPVariable;
    reactions.reactionBoolVariableIndex(i) = lastVariableIndex;
end

function reaction_equations()

global reactions;
global lastVariableIndex;

for i=1:length(reactions.reactionName)
    rule_data_struct = reactions.reactionRuleDS{i};
    lastVariableIndex = ConvertRuleToEquations(rule_data_struct, 'reaction_mode', i, lastVariableIndex);   
end

function reaction_bounds_equations()

global reactions;

% Relate between reaction boolean value and the reaction constrains - (view document 'How to build the MIP problem.doc - Section B.c')
% For each reaction we need to add those equation to:
%   1.	V  Y*Vmax <= 0
%   2.	0 <= V - Y*Vmin

for i=1:length(reactions.reactionName)
    vmin = reactions.reactionConstrainsLow(i);
    vmax = reactions.reactionConstrainsHigh(i);

    %   V  Y*Vmax <= 0
    % The reaction index is also the index of the reaction variable in the matrix 
    colIndices = [i reactions.reactionBoolVariableIndex(i)];
    colValues = [1 -vmax];
    addMIPEquation(colIndices, colValues, -inf, 0);    
    
    %   0 <= V - Y*Vmin
    % The reaction index is also the index of the reaction variable in the matrix 
    colIndices = [i reactions.reactionBoolVariableIndex(i)];
    colValues = [1 -vmin];
    addMIPEquation(colIndices, colValues, 0, inf);    
end

function addBooleanMIPVariable()

global mipVariableType;
global mipColumnLB;
global mipColumnUB;

mipColumnLB = [mipColumnLB ; 0];
mipColumnUB = [mipColumnUB ; 1];
mipVariableType = [mipVariableType 'B']; % Define this variable as a binary
    
function [state] = GetCellStateState(cellstate_str)
% returns the value\State (0,1) of the Cell state with the string 'cellstate_str'

global cellStates;

cellstate_str = strrep(cellstate_str,'"','');

for i=1:length(cellStates.cellStateName)
    if (strcmp(cellstate_str, cellStates.cellStateName(i)))
        state = cellStates.cellStateState(i);
        return;
    end
end

% If no gene was found, return error state
state = -1;

function newEquationIndex = addMIPEquation(colIndices, colValues, lowerBound, upperBound)
% If the indices in colIndices are greater than the mipMatrix size, the matrix is expanded
global mipMatrix;
global mipRowLB;
global mipRowUB;

matrixSize = size(mipMatrix);
newEquationIndex = matrixSize(1)+1;

for i=1:length(colIndices)
    % matrixSize(1)+1 is the index of the new Row
    mipMatrix(newEquationIndex, colIndices(i)) = colValues(i);
end

mipRowLB = [mipRowLB ; lowerBound];
mipRowUB = [mipRowUB ; upperBound];

% this function is responsible to convert the gene rule into a series of MIP equations and set the matrix for that.
function [lastVariableIndex] = ConvertRuleToEquations(rule_data_struct, element_type, element_index, lastVariableIndex)
% element_type is one of: 'gene_mode', 'protein_mode', 'reaction_mode'
% [newLastVariableIndex, firstGeneEquationIndex, lastGeneEquationIndex]
global genes;
global proteins;
global reactions;
global PH;

epsilon = 0.001;

% initialize the indices vector of the rule DS booleans to zero - it doesn't really matters what is the value
% The following vector can contain one of 3 options:
%   1.  The value -1 - represent a '1' boolean result of the mid expression
%   2.  The value -2 - represent a '0' boolean result of the mid expression
%   3.  The value -3 - represent a 'not-determind' state
%   4.  An index that represent a boolean variable from the solver matrix that this correspond with this mid expression
ruleDS_indices = zeros(length(rule_data_struct),1);

for i = 1:length(rule_data_struct)
        
        entry = rule_data_struct{i};

        if (strcmp( entry.operator , 'NAME'))            
            % A name can be one of 3:
            % 1.    The string 'on' or 'off'. 
            % 2.    A gene name.
            % 3.    A cell state name.            
            if (strcmp(lower(entry.param1),'on'))
                ruleDS_indices(i) = -1;
            else
                if (strcmp(lower(entry.param1),'off'))
                    ruleDS_indices(i) = -2;
                else
                    if (strcmp(lower(entry.param1),'not-determind'))
                        ruleDS_indices(i) = -3;
                    else
                        % Check source
                        sourceIndex = str2num(entry.param1);
                        if (~isempty(sourceIndex))
                            if ((strcmp(element_type,'gene_mode')) | (strcmp(element_type,'protein_mode')))
                                ruleDS_indices(i) = genes.geneBoolVariableIndex(sourceIndex);
                            else % strcmp(element_type,'reaction_mode')
                                ruleDS_indices(i) = proteins.proteinBoolVariableIndex(sourceIndex);
                            end
                        else
                            % In this case we need to check if it's a cell state
                            cellState = GetCellStateState(entry.param1);
                            if (cellState >= 0)
                                % In this case, the value is a Cell state and I copy it's value to the entry
                                if (cellState == 0)
                                    ruleDS_indices(i) = -2;
                                else
                                    ruleDS_indices(i) = -1;
                                end
                            else
                                % In this case no good meaning of the entry 'NAME' was found - Throw a warning
                                warningStr = sprintf('AddGeneEquations - The entry %d could not be evaluated as a "NAME". Param = %s',i,entry.param1);
                                warning(warningStr);
                                ret_eval = -1;
                                return;
                            end
                        end
                    end
                end;
            end;                                                 
        else            
        if (strcmp( entry.operator , '>'))
            % A '>' can be one of 3:
            % 1.    A flux.
            % 2.    Growth > 0.
            % 3.    PH > val. 
            fluxIndex = str2num(entry.param1);                                
            compareVal = str2num(entry.param2);          
            if (~isempty(fluxIndex))
                % Here I translate the meaning of constarains from the Solver equations to constrains in the meaing of the 
                % regulatory rules. 
                % This means that if the rule asked if (flux > val), the real question is if (flux < -val).
                % Two equations that are added (view document 'How to build the MIP problem.doc'):
                %       1.	c * (Vmax - const) + v <= Vmax
                %       2.	const <= c * (const - Vmin) + v
                
                vmax = reactions.reactionConstrainsHigh(fluxIndex);
                vmin = reactions.reactionConstrainsLow(fluxIndex);
                % I also remove the epsilon becuase there is no < (no <=) in the solver
                compareVal = -compareVal-epsilon;          
                
                % The following 6 lines should be marked in case I want to add Mutants
                if (compareVal <= vmin)
                    ruleDS_indices(i) = -2;
                else
                if (compareVal > vmax)
                    ruleDS_indices(i) = -1;
                else
                    % Add one more variable 'c' of the equations
                    lastVariableIndex = lastVariableIndex + 1;
                    addBooleanMIPVariable;

                    % First equation
                    colIndices = [lastVariableIndex fluxIndex];
                    colValues = [(vmax-compareVal) 1];
                    addMIPEquation(colIndices, colValues, -inf, vmax);                                                           
                    
                    % Second equation
                    colIndices = [lastVariableIndex fluxIndex];
                    colValues = [(compareVal-vmin) 1];
                    addMIPEquation(colIndices, colValues, compareVal, inf);
                    
                    ruleDS_indices(i) = lastVariableIndex;
                end;end;
                
            else
                if (strcmp(lower(entry.param1),'growth'))
                    % assume Biomass reaction is positive and that the Rule is 'Growth > 0'
                    ruleDS_indices(i) = -1;
                else
                    if (strcmp(lower(entry.param1),'ph'))
                        if (PH > compareVal)
                            ruleDS_indices(i) = -1;
                        else
                            ruleDS_indices(i) = -2;
                        end
                            
                    else
                        % In this case no flux was found - It means that we can't evaluate
                        warningStr = sprintf('AddGeneEquations - ">" sign - The entry %d could not be evaluated as a flux. Param1 = %s, Param2 = %s',i,entry.param1,entry.param2);
                        warning(warningStr);
                        ret_eval = -1;
                        return;
                    end
                end
            end
        else
        if (strcmp( entry.operator , '<'))
            % A '<' can be one of 3:
            % 1.    A flux.
            % 2.    Growth < 0.
            % 3.    PH < val.
            fluxIndex = str2num(entry.param1);
            compareVal = str2num(entry.param2);
            if (~isempty(fluxIndex))
                % Here I translate the meaning of constarains from the GLPK equations to constrains in the meaing of the 
                % regulatory rules. 
                % This means that if the rule asked if (flux < val), the real question is if (flux > -val).
                % Two equations that are added (view document 'How to build the MIP problem.doc'):
                %       1.	c * (const - Vmax) + v <= const
                %       2.	Vmin <= c * (Vmin - const) + v
                
                vmax = reactions.reactionConstrainsHigh(fluxIndex);
                vmin = reactions.reactionConstrainsLow(fluxIndex);
                % I also add the epsilon becuase there is no < (no <=) in the solver
                compareVal = -compareVal+epsilon;          

                % The following 6 lines should be marked in case I want to add Mutants
                if (compareVal < vmin)
                    ruleDS_indices(i) = -1;
                else
                if (compareVal >= vmax)
                    ruleDS_indices(i) = -2;
                else
                    % Add one more variable 'c' from the equations
                    lastVariableIndex = lastVariableIndex + 1;
                    addBooleanMIPVariable;
                    
                    % First equation
                    colIndices = [lastVariableIndex fluxIndex];
                    colValues = [(compareVal-vmax) 1];
                    addMIPEquation(colIndices, colValues, -inf, compareVal);
                
                    % Second equation
                    colIndices = [lastVariableIndex fluxIndex];
                    colValues = [(vmin-compareVal) 1];
                    addMIPEquation(colIndices, colValues, vmin, inf);
                    
                    ruleDS_indices(i) = lastVariableIndex;
                end;end;

            else
                if (strcmp(lower(entry.param1),'growth'))
                    % assume Biomass reaction is positive and that the Rule is 'Growth < 0'
                    ruleDS_indices(i) = -2;
                else
                    if (strcmp(lower(entry.param1),'ph'))
                        if (PH < compareVal)
                            ruleDS_indices(i) = -1;
                        else
                            ruleDS_indices(i) = -2;
                        end
                    else
                        % In this case no flux was found - It means that we can't evaluate
                        warningStr = sprintf('AddGeneEquations - ">" sign - The entry %d could not be evaluated as a flux. Param1 = %s, Param2 = %s',i,entry.param1,entry.param2);                        
                        warning(warningStr);
                        ret_eval = -1;
                        return;
                    end
                end
            end
        else
        if (strcmp( entry.operator , 'OR'))
            leftEntryNum = sscanf(entry.param1,'entry%d');
            rightEntryNum = sscanf(entry.param2,'entry%d');
            
            if ((ruleDS_indices(leftEntryNum) == -1) | (ruleDS_indices(rightEntryNum) == -1))
                ruleDS_indices(i) = -1;
            else 
            if ((ruleDS_indices(leftEntryNum) == -2) & (ruleDS_indices(rightEntryNum) == -2))
                ruleDS_indices(i) = -2;
            else
                if (ruleDS_indices(leftEntryNum) == -2) % the other entry can't be (-1) nor (-2), it must be an index
                    ruleDS_indices(i) = ruleDS_indices(rightEntryNum);
                else 
                if (ruleDS_indices(rightEntryNum) == -2) % the other entry can't be (-1) nor (-2), it must be an index
                    ruleDS_indices(i) = ruleDS_indices(leftEntryNum);
                else
                    % both must be indices
                    % In this case we need to build the following equation - (view document 'How to build the MIP problem.doc'):
                    % a = (b or c)  is converted to (-2 <= 2*b + 2 *c  4*a <= 1)            
                    % Add one more variable 'a' from the equations
                    lastVariableIndex = lastVariableIndex + 1;                    
                    
                    addBooleanMIPVariable;
    
                    colIndices = [ruleDS_indices(leftEntryNum) ruleDS_indices(rightEntryNum) lastVariableIndex];
                    colValues = [2 2 -4];
                    addMIPEquation(colIndices, colValues, -2, 1);                                
                    
                    ruleDS_indices(i) = lastVariableIndex;
                end;end;
            end;end;
        else
        if (strcmp( entry.operator , 'AND'))
            leftEntryNum = sscanf(entry.param1,'entry%d');
            rightEntryNum = sscanf(entry.param2,'entry%d');

            if ((ruleDS_indices(leftEntryNum) == -2) | (ruleDS_indices(rightEntryNum) == -2))
                ruleDS_indices(i) = -2;
            else 
            if ((ruleDS_indices(leftEntryNum) == -1) & (ruleDS_indices(rightEntryNum) == -1))
                ruleDS_indices(i) = -1;
            else
                if (ruleDS_indices(leftEntryNum) == -1) % the other entry can't be (-1) nor (-2), it must be an index
                    ruleDS_indices(i) = ruleDS_indices(rightEntryNum);
                else 
                if (ruleDS_indices(rightEntryNum) == -1) % the other entry can't be (-1) nor (-2), it must be an index
                    ruleDS_indices(i) = ruleDS_indices(leftEntryNum);
                else
                    % both must be indices
                    % In this case we need to build the following equation - (view document 'How to build the MIP problem.doc'):
                    % a = (b and c) is converted to  (-1 <= 2*b + 2*c  4*a <= 3)
                    % Add one more variable 'a' from the equations
                    lastVariableIndex = lastVariableIndex + 1;
                    addBooleanMIPVariable;
                
                    colIndices = [ruleDS_indices(leftEntryNum) ruleDS_indices(rightEntryNum) lastVariableIndex];
                    colValues = [2 2 -4];
                    addMIPEquation(colIndices, colValues, -1, 3);
                    
                    ruleDS_indices(i) = lastVariableIndex;
                end;end;
            end;end;
        
        else
        if (strcmp( entry.operator , 'NOT'))
            leftEntryNum = sscanf(entry.param1,'entry%d');

            if (ruleDS_indices(leftEntryNum) == -2)
                ruleDS_indices(i) = -1;
            else
            if (ruleDS_indices(leftEntryNum) == -1)
                ruleDS_indices(i) = -2;
            else
                % In this case we need to build the following equation - (view document 'How to build the MIP problem.doc'):
                % a = not (b) is converted to  (b + a = 1) 
                % Add one more variable 'a' from the equations
                lastVariableIndex = lastVariableIndex + 1;
                addBooleanMIPVariable;
                
                colIndices = [ruleDS_indices(leftEntryNum) lastVariableIndex];
                colValues = [1 1];
                addMIPEquation(colIndices, colValues, 1, 1);

                ruleDS_indices(i) = lastVariableIndex;
            end;end;                
        else
            errorStr = sprintf('AddGeneEquations - Entry number %d has a wrong type - %s',i,entry.operator);
            error(errorStr);
        end;end;end;end;end;end;   
end

if (strcmp(element_type,'gene_mode'))
    elementVariableIndex = genes.geneBoolVariableIndex(element_index);
else if (strcmp(element_type,'protein_mode'))
    elementVariableIndex = proteins.proteinBoolVariableIndex(element_index);
else if (strcmp(element_type,'reaction_mode'))
    elementVariableIndex = reactions.reactionBoolVariableIndex(element_index);
end;end;end

% Set an equation that relates the gene boolean variable with its Rule
if (ruleDS_indices(length(rule_data_struct)) == -1) % meaning the rule is '1'
    colIndices = [elementVariableIndex];
    colValues = [1];
    addMIPEquation(colIndices, colValues, 1, 1);
else
if (ruleDS_indices(length(rule_data_struct)) == -2) % meaning the rule is '0'
    colIndices = [elementVariableIndex];
    colValues = [1];
    addMIPEquation(colIndices, colValues, 0, 0);
else
if (ruleDS_indices(length(rule_data_struct)) == -3) % meaning the rule is 'not_determind'        
    colIndices = [elementVariableIndex];
    colValues = [1];
    addMIPEquation(colIndices, colValues, 0, 1);
else   
    % Ensure that the gene boolean variable is equal to the result of the regulatory rule

    colIndices = [elementVariableIndex ruleDS_indices(length(rule_data_struct))];
    colValues = [1 -1];
    addMIPEquation(colIndices, colValues, 0, 0);
end;end;end;

function status = CreateCPlexInputFile(fileName, objective_element, objective_index, maximize_or_minimize)

global mipMatrix;
global mipRowLB;
global mipRowUB;
global mipColumnLB;
global mipColumnUB;
global mipVariableType;
global genes;
global proteins;
global reactions;

outFile = fopen(fileName,'w');

% Objective function
if (strcmp(maximize_or_minimize,'maximize'))
    fprintf(outFile, 'Maximize\n');
else
    fprintf(outFile, 'Minimize\n');
end

if (strcmp(objective_element,'gene'))
    if (objective_index > length(genes.geneBoolVariableIndex))
        str = sprintf('objective_index %d for type gene exceeded the allowed value of %d',objective_index, length(genes.geneBoolVariableIndex));
        error(str);
    else
        objective_index = genes.geneBoolVariableIndex(objective_index);
    end
else
if (strcmp(objective_element,'protein'))
    if (objective_index > length(proteins.proteinBoolVariableIndex))
        str = sprintf('objective_index %d for type protein exceeded the allowed value of %d',objective_index, length(proteins.proteinBoolVariableIndex));
        error(str);
    else
        objective_index = proteins.proteinBoolVariableIndex(objective_index);
    end
else
if (strcmp(objective_element,'reaction'))
    if (objective_index > length(reactions.reactionName))
        str = sprintf('objective_index %d for type reaction exceeded the allowed value of %d',objective_index, length(reactions.reactionName));
        error(str);
    else
        % For reaction, the reaction index is also the reactions variable index
        objective_index = objective_index;
    end
end;end;end;

fprintf(outFile, ' obj: x%d \n', objective_index);

% Constrains
fprintf(outFile, 'Subject to\n');
matrix_size = size(mipMatrix);
counter = 1;
for i=1:matrix_size(1)
    equation_middle = '';
    non_zero_indices = find(mipMatrix(i,:));
    for j=1:length(non_zero_indices)
        val = mipMatrix(i,non_zero_indices(j));
        if (val > 0)
            equation_middle = sprintf('%s +',equation_middle);
        end        
        equation_middle = sprintf('%s %d x%d', equation_middle, val, non_zero_indices(j));
    end
        
    fprintf(outFile, 'c%d: ',counter);
    counter = counter + 1;
    if (mipRowUB(i) == Inf)
        fprintf(outFile,'%s <= 1.0E+20\n',equation_middle);
    else
        fprintf(outFile,'%s <= %f\n',equation_middle, mipRowUB(i));
    end
    
    fprintf(outFile, 'c%d: ',counter);
    counter = counter + 1;
    if (mipRowLB(i) == -Inf)    
        fprintf(outFile,'%s >= -1.0E+20\n',equation_middle);
    else
        fprintf(outFile,'%s >= %f\n',equation_middle, mipRowLB(i));
    end
    
end

% Bounds
fprintf(outFile, 'Bounds\n');
for i=1:length(mipColumnLB)
    fprintf(outFile, '%d <= x%d <= %d\n',mipColumnLB(i), i, mipColumnUB(i));
end

% variable type
continues = find(mipVariableType == 'C');
binaries = find(mipVariableType == 'B');

fprintf(outFile, 'Binary\n');
for i=1:length(binaries)
    fprintf(outFile, 'x%d\n',binaries(i));
end

% The end
fprintf(outFile, 'End');
fclose(outFile);

function PrintProblemStats()

global genes;
global proteins;
global reactions;
global mipMatrix;

sz = size(mipMatrix);

str = sprintf('Problem has %d rows with %d variables',sz(1), sz(2));
disp(str);

disp('Result vector is built accordingly:');

str = sprintf('1.   Gene boolean vector starts from index %d to index %d ,',genes.geneBoolVariableIndex(1),genes.geneBoolVariableIndex(end));
disp(str);
disp('     according to the gene indices in "iMC1010v1.xls" or "iMC1010v2.xls" from http://gcrg.ucsd.edu/organisms/ecoli/regulation');
disp(' ');
str = sprintf('2.   Protein boolean vector starts from index %d to index %d ,',proteins.proteinBoolVariableIndex(1),proteins.proteinBoolVariableIndex(end));
disp(str);
disp('     according to the protein indices in "iJR904(7-28-04).xls" from http://gcrg.ucsd.edu/organisms/ecoli/metabrxns/iJR904(7-28-04).xls');
disp(' ');
str = sprintf('3.   Reaction boolean vector starts from index %d to index %d ,',reactions.reactionBoolVariableIndex(1),reactions.reactionBoolVariableIndex(end));
disp(str);
disp('     according to the reaction indices in "iJR904(7-28-04).xls" from http://gcrg.ucsd.edu/organisms/ecoli/metabrxns/iJR904(7-28-04).xls');
disp(' ');
str = sprintf('4.   Reaction flux values starts from index 1 to index %d',length(reactions.reactionName));
disp(str);